SystemOrganization addCategory: #Maybe!
SystemOrganization addCategory: #'Maybe-Tests'!

TestCase subclass: #JustTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Maybe-Tests'!

!JustTest methodsFor: 'testing - monad' stamp: 'fbs 6/28/2011 22:42'!
testBindReturnsJustWithFunctionApplied
	| j |
	j := (Just value: 2) >>= [:x | x * 2].
	self assert: j hasValue.
	self assert: j value = 4.! !

!JustTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:33'!
testIfNothingIfNotNothingRunsJustBlock
	self assert: 2 == ((Maybe value: 42) ifNothing: [1] ifNotNothing: [2])! !

!JustTest methodsFor: 'testing - running' stamp: 'fbs 6/15/2011 17:52'!
testMaybeDefaultReturnsBlockValue
	| block default result thing |
	default := Object new.
	block := [:x | x * 2].
	thing := Maybe value: 2.
	result := thing maybe: block default: default.
	self assert: result = (block value: thing value).! !

!JustTest methodsFor: 'testing - printing' stamp: 'fbs 6/15/2011 17:41'!
testPrintStringLooksLikeHaskell
	self assert: (Just value: 2) printString = 'Just 2'.! !

TestCase subclass: #MaybeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Maybe-Tests'!

!MaybeTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:27'!
testNothingHasNoValue
	self deny: Maybe nothing hasValue.
	
	self should: [Maybe nothing value] raise: Error.! !

!MaybeTest methodsFor: 'testing - instance creation' stamp: 'fbs 6/15/2011 17:26'!
testNothingIsIdempotent
	self assert: Maybe nothing == Maybe nothing.! !

!MaybeTest methodsFor: 'testing - monad' stamp: 'fbs 6/28/2011 22:15'!
testReturnAddsObjectToMonad
	| m |
	m := Maybe return: 1.
	self assert: m hasValue.! !

!MaybeTest methodsFor: 'testing - instance creation' stamp: 'fbs 6/15/2011 17:24'!
testValueReturnsaJust
	| r |
	r := Maybe value: 1.
	self assert: r hasValue.! !

TestCase subclass: #NothingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Maybe-Tests'!

!NothingTest methodsFor: 'testing - monad' stamp: 'fbs 6/28/2011 22:29'!
testBindReturnsNothing
	self deny: (Maybe nothing >>= [:x | x * 2]) hasValue! !

!NothingTest methodsFor: 'testing - testing' stamp: 'fbs 6/15/2011 17:34'!
testIfNothingIfNotNothingRunsNothingBlock
	self assert: 1 == (Maybe nothing ifNothing: [1] ifNotNothing: [2]).
	self assert: 1 == ((Nothing new) ifNothing: [1] ifNotNothing: [2]).! !

!NothingTest methodsFor: 'testing - applying' stamp: 'fbs 6/15/2011 17:55'!
testMaybeDefaultReturnsDefault
	| block default result |
	default := Object new.
	block := [:x | x * 2].
	result := Maybe nothing maybe: block default: default.
	self assert: result = default.! !

!NothingTest methodsFor: 'testing - printing' stamp: 'fbs 6/15/2011 17:40'!
testPrintStringLooksLikeHaskell
	self assert: Maybe nothing printString = 'Nothing'.! !

Object subclass: #Maybe
	instanceVariableNames: ''
	classVariableNames: 'NothingSingleton'
	poolDictionaries: ''
	category: 'Maybe'!

Maybe subclass: #Just
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Maybe'!

!Just class methodsFor: 'private' stamp: 'fbs 6/14/2011 18:09'!
value: anObject
	^ self new value: anObject.! !

!Just methodsFor: 'testing' stamp: 'fbs 6/15/2011 17:24'!
hasValue
	^ true.! !

!Just methodsFor: 'applying' stamp: 'fbs 6/15/2011 17:56'!
maybe: aBlock default: anObject
	^ aBlock value: value.! !

!Just methodsFor: 'printing' stamp: 'fbs 6/15/2011 17:42'!
printOn: aStream
	aStream nextPutAll: 'Just '.
	^ value printOn: aStream.! !

!Just methodsFor: 'accessing' stamp: 'fbs 6/14/2011 18:09'!
value
	^ value! !

!Just methodsFor: 'private' stamp: 'fbs 6/14/2011 18:10'!
value: anObject
	value := anObject.! !

!Maybe class methodsFor: 'class initialization' stamp: 'fbs 6/14/2011 18:38'!
initialize
	^ NothingSingleton ifNil: [NothingSingleton := Nothing new]! !

!Maybe class methodsFor: 'instance creation' stamp: 'fbs 6/14/2011 18:38'!
nothing
	^ NothingSingleton! !

!Maybe class methodsFor: 'instance creation' stamp: 'fbs 6/28/2011 22:22'!
return: anObject
	^ Just value: anObject.! !

!Maybe class methodsFor: 'instance creation' stamp: 'fbs 6/14/2011 18:09'!
value: anObject
	^ Just value: anObject.! !

!Maybe methodsFor: 'monad' stamp: 'fbs 6/28/2011 22:26'!
>>= aUnaryBlock
	self subclassResponsibility.! !

!Maybe methodsFor: 'testing' stamp: 'fbs 6/15/2011 17:24'!
hasValue
	^ false.! !

!Maybe methodsFor: 'testing' stamp: 'fbs 6/15/2011 17:31'!
ifNothing: nothingBlock ifNotNothing: justBlock
	^ self hasValue
		ifTrue: [justBlock value]
		ifFalse: [nothingBlock value]! !

!Maybe methodsFor: 'applying' stamp: 'fbs 6/21/2011 20:20'!
maybe: aBlock default: anObject
	self subclassResponsibility! !

!Maybe methodsFor: 'accessing' stamp: 'fbs 6/14/2011 18:10'!
value
	self subclassResponsibility! !

Maybe subclass: #Nothing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Maybe'!

!Nothing methodsFor: 'monad' stamp: 'fbs 6/28/2011 22:27'!
>>= aUnaryBlock
	^ Maybe nothing.! !

!Nothing methodsFor: 'applying' stamp: 'fbs 6/15/2011 17:55'!
maybe: aBlock default: anObject
	^ anObject.! !

!Nothing methodsFor: 'printing' stamp: 'fbs 6/15/2011 17:40'!
printOn: aStream
	^ aStream nextPutAll: 'Nothing'.! !

!Nothing methodsFor: 'accessing' stamp: 'fbs 6/14/2011 18:10'!
value
	Error signal: 'None has no value'.! !
Maybe initialize!
